package org.briarproject.bramble.api.contact;

import org.briarproject.bramble.api.FormatException;
import org.briarproject.bramble.api.nullsafety.NotNullByDefault;

import javax.annotation.concurrent.Immutable;

/**
 * Enumeration of the different contact types.
 */
@Immutable
@NotNullByDefault
public enum ContactType {

	/**
	 * A regular contact.
	 */
	CONTACT(0),
	/**
	 * A mailbox paired with this device.
	 */
	PRIVATE_MAILBOX(1),
	/**
	 * The owner of a mailbox.
	 */
	MAILBOX_OWNER(2),

	/**
	 * A contact's mailbox
	 */
	CONTACT_MAILBOX(3),

	/**
	 * A contact of the mailbox owner connected to the mailbox
	 */
	MAILBOX_CONTACT(4);

	private final int value;

	ContactType(int value) {
		this.value = value;
	}

	// Use this for deserialization
	public static ContactType fromValue(int value)
			throws FormatException {
		for (ContactType type : values()) if (type.value == value) return type;
		throw new FormatException();
	}

	// Use this for values coming from the database
	public static ContactType fromLocalValue(int value) {
		for (ContactType type : values()) if (type.value == value) return type;
		throw new IllegalArgumentException();
	}

	public int getValue() {
		return value;
	}
}
