package org.briarproject.bramble.mailbox.protocol;


import java.net.ProtocolException;

public interface MailboxRequestHandler {
	/**
	 * Called if a request is received which matches the {@link MailboxMessage.TYPE} of the
	 * {@link MailboxRequestHandler#getType()} return
	 *
	 * @param request
	 * @throws ProtocolException If this function throws a Protocol exception,
	 * an error response will be sent
	 * NOTE: Ensure that the ProtocolException does not include sensitive information
	 */
	void handleRequest(MailboxRequest request) throws ProtocolException;

	/**
	 * @return Must return the {@link MailboxMessage.TYPE} which can be handled by this handler
	 */
	MailboxMessage.TYPE getType();

	/**
	 * Is called when the protocol is finished (once the underlying connection is closed)
	 */
	void protocolFinished();
}
