package org.briarproject.bramble.mailbox.protocol;

import org.briarproject.bramble.api.FormatException;
import org.briarproject.bramble.api.contact.ContactId;
import org.briarproject.bramble.api.data.BdfList;

import javax.annotation.Nullable;

public class MailboxRequestTake extends MailboxRequest {
	private final ContactId contactId;
	private final byte[] encryptedSyncStream;

	public MailboxRequestTake(byte[] encryptedSyncStream) {
		super(TYPE.TAKE);
		this.contactId = null;
		this.encryptedSyncStream = encryptedSyncStream;
	}

	public MailboxRequestTake(ContactId contactId, byte[] encryptedSyncStream) {
		super(TYPE.TAKE);
		this.contactId = contactId;
		this.encryptedSyncStream = encryptedSyncStream;
	}

	public MailboxRequestTake(BdfList msg) throws FormatException {
		super(TYPE.TAKE, msg.getLong(1));

		BdfList body = msg.getList(2);
		if (body.size() != 2)
			throw new FormatException();

		Long cId = body.getOptionalLong(0);
		if (cId != null) {
			if (cId > Integer.MAX_VALUE || cId < Integer.MIN_VALUE)
				throw new FormatException();
			contactId = new ContactId(cId.intValue());
		} else
			contactId = null;

		encryptedSyncStream = body.getRaw(1);
	}

	@Override
	protected BdfList getRequestBody() {
		if (contactId == null)
			return BdfList.of(null, encryptedSyncStream);

		return BdfList.of(contactId.getInt(), encryptedSyncStream);
	}

	@Override
	public boolean hasResponse() {
		return true;
	}

	public byte[] getEncryptedSyncStream() {
		return encryptedSyncStream;
	}

	@Nullable
	public ContactId getContactId() {
		return contactId;
	}

	public boolean hasContactId() {
		return contactId != null;
	}
}
