package org.briarproject.bramble.mailbox.protocol;

import org.briarproject.bramble.api.FormatException;
import org.briarproject.bramble.api.data.BdfList;

import javax.annotation.Nullable;

import static org.briarproject.bramble.mailbox.protocol.MailboxMessage.TYPE.RESPONSE;

public class MailboxResponse implements MailboxMessage {
	@Nullable
	private String errorMessage;
	private long msgId;

	public MailboxResponse(long msgId, @Nullable String errorMessage) {
		this.msgId = msgId;
		this.errorMessage = errorMessage;
	}

	public MailboxResponse(BdfList msg) throws FormatException {
		this.msgId = msg.getLong(1);
		BdfList body = msg.getList(2);

		if(body.size() != 1)
			throw new FormatException();

		errorMessage = body.getOptionalString(0);
	}

	@Override
	public long getId() {
		return msgId;
	}

	@Override
	public TYPE getType() {
		return RESPONSE;
	}

	@Override
	public BdfList toBdfList() {
		return BdfList.of(RESPONSE.getValue(), msgId, BdfList.of(errorMessage));
	}

	public String getErrorMessage() {
		return errorMessage;
	}

	public boolean isSuccess() {
		return errorMessage == null;
	}
}