package org.briarproject.bramble.mailbox.sessions;

import java.io.ByteArrayOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;

public class FeedableSyncInputStream extends InputStream {
	private ByteArrayOutputStream os = new ByteArrayOutputStream();
	private boolean isEOF = false;
	private byte[] activeBuffer = new byte[]{};
	private int activeBufferPointer = 0;

	@Override
	public synchronized int read() throws IOException {
		while (!hasBytes()) {
			if(isEOF)
				throw new EOFException();

			try {
				this.wait();
			} catch (InterruptedException e) {
				throw new IOException(e.toString());
			}
		}

		return activeBuffer[activeBufferPointer++] & 0xff;
	}

	private boolean hasBytes() {
		// Check if active buffer has bytes left
		if (activeBufferPointer < activeBuffer.length)
			return true;

		// check if bytes from OS are available
		if (os.size() <= 0)
			return false;

		// update active buffer if bytes from OS are available
		activeBuffer = os.toByteArray();
		os.reset();
		activeBufferPointer = 0;
		return true;
	}

	public synchronized void feed(byte[] buffer) throws IOException {
		if (isEOF)
			throw new EOFException();

		os.write(buffer);

		this.notifyAll();
	}

	public synchronized void close() {
		isEOF = true;
		this.notifyAll();
	}

	public synchronized boolean isClosed() {
		return isEOF;
	}


}
