package org.briarproject.bramble.mailbox.sessions;

import org.briarproject.bramble.api.contact.ContactId;
import org.briarproject.bramble.api.contact.ContactType;
import org.briarproject.bramble.api.db.DatabaseComponent;
import org.briarproject.bramble.api.lifecycle.IoExecutor;
import org.briarproject.bramble.api.mailbox.MailboxStorage;
import org.briarproject.bramble.api.sync.SyncSessionFactory;
import org.briarproject.bramble.api.transport.KeyManager;
import org.briarproject.bramble.api.transport.StreamReaderFactory;
import org.briarproject.bramble.api.transport.StreamWriterFactory;
import org.briarproject.bramble.mailbox.protocol.MailboxProtocol;
import org.briarproject.bramble.mailbox.storage.MailboxStorageImpl;

import java.util.concurrent.Executor;

import javax.inject.Inject;

public class MailboxSessionFactoryImpl implements MailboxSessionFactory {

	private MailboxStorage mailboxStorage = null;
	private Executor ioExecutor;
	private final KeyManager keyManager;
	private final SyncSessionFactory syncSessionFactory;
	private final StreamWriterFactory streamWriterFactory;
	private final StreamReaderFactory streamReaderFactory;
	private DatabaseComponent db;

	@Inject
	public MailboxSessionFactoryImpl(@IoExecutor Executor ioExecutor,
			KeyManager keyManager,
			SyncSessionFactory syncSessionFactory,
			StreamWriterFactory streamWriterFactory,
			StreamReaderFactory streamReaderFactory, DatabaseComponent db) {
		this.ioExecutor = ioExecutor;
		this.keyManager = keyManager;
		this.syncSessionFactory = syncSessionFactory;
		this.streamWriterFactory = streamWriterFactory;
		this.streamReaderFactory = streamReaderFactory;
		this.db = db;

		// FIXME: using temporary storage for now
		mailboxStorage = new MailboxStorageImpl();
	}

	@Override
	public AbstractMailboxSession createMailboxSession(
			MailboxProtocol mailboxProtocol,
			ContactId contactId, ContactType contactType,
			int transportMaxLatency,
			int transportMaxIdleTime) {
		switch (contactType) {
			case CONTACT:
				throw new RuntimeException(
						"A contact of type CONTACT can not be handled by a mailbox session");
			case PRIVATE_MAILBOX:
				return new PrivateMailboxSession(contactId, ioExecutor,
						keyManager, syncSessionFactory, streamWriterFactory,
						streamReaderFactory, mailboxProtocol,
						transportMaxLatency, transportMaxIdleTime, db);
			case CONTACT_MAILBOX:
				return new ContactMailboxSession(contactId, ioExecutor,
						keyManager, syncSessionFactory, streamWriterFactory,
						streamReaderFactory, mailboxProtocol,
						transportMaxLatency, transportMaxIdleTime, db);
			case MAILBOX_OWNER:
				return new MailboxOwnerSession(contactId, ioExecutor,
						keyManager, syncSessionFactory, streamWriterFactory,
						streamReaderFactory, mailboxProtocol,
						transportMaxLatency, transportMaxIdleTime,
						mailboxStorage, db);
			case MAILBOX_CONTACT:
				return new MailboxContactSession(contactId, ioExecutor,
						keyManager, syncSessionFactory, streamWriterFactory,
						streamReaderFactory, mailboxProtocol,
						transportMaxLatency, transportMaxIdleTime,
						mailboxStorage, db);
		}
		return null;
	}
}
