package org.briarproject.bramble.mailbox.sessions;

import org.briarproject.bramble.api.transport.StreamWriter;
import org.briarproject.bramble.mailbox.protocol.MailboxProtocol;
import org.briarproject.bramble.mailbox.protocol.MailboxRequestSync;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.concurrent.atomic.AtomicBoolean;

class MailboxSyncRequestWriter extends OutputStream implements StreamWriter {

	private MailboxProtocol mailboxProtocol;
	private final ByteArrayOutputStream bufferOS = new ByteArrayOutputStream();
	private boolean endOfStream = false;

	public MailboxSyncRequestWriter(MailboxProtocol mailboxProtocol) {
		this.mailboxProtocol = mailboxProtocol;
	}

	@Override
	public OutputStream getOutputStream() {
		return this;
	}

	@Override
	public synchronized void sendEndOfStream() throws IOException {
		if (endOfStream)
			throw new IOException("End of stream was already written");

		MailboxRequestSync req = new MailboxRequestSync();
		try {
			mailboxProtocol.writeRequest(req);
		} catch (InterruptedException e) {
			throw new IOException(e.toString());
		}
	}

	@Override
	public synchronized void write(int i) {
		bufferOS.write(i);
	}

	@Override
	public synchronized void flush() throws IOException {
		if (endOfStream)
			throw new IOException("End of stream already written");

		// If a sync session flushes without data to write it indicates
		// a keep alive
		if (bufferOS.size() == 0){
			mailboxProtocol.sendKeepAlive();
			return;
		}

		byte[] syncStream = bufferOS.toByteArray();
		bufferOS.reset();

		MailboxRequestSync req = new MailboxRequestSync(syncStream);
		try {
			mailboxProtocol.writeRequest(req);
		} catch (InterruptedException e) {
			throw new IOException(e.toString());
		}
	}
}
