package org.briarproject.bramble.mailbox.storage;

import org.briarproject.bramble.api.contact.ContactId;
import org.briarproject.bramble.api.mailbox.MailboxStorage;
import org.briarproject.bramble.api.mailbox.MailboxStorageStream;

import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.concurrent.atomic.AtomicLong;
import java.util.logging.Logger;

import javax.annotation.Nullable;

public class MailboxStorageImpl implements MailboxStorage {
	private static final Logger LOG =
			Logger.getLogger(MailboxStorageImpl.class.getName());
	static volatile AtomicLong streamCounter = new AtomicLong();

	// Dummy implementation with a hashmap for now
	LinkedHashMap<ContactId, LinkedList<MailboxStorageStream>> contactStreams =
			new LinkedHashMap();

	public 	synchronized void storeStream(ContactId contactId, byte[] encryptedStream,
			boolean forOwner) {

		LOG.info("Storing STREAM");
		MailboxStorageStream stream =
				new MailboxStorageStreamImpl(contactId, encryptedStream,
						streamCounter.incrementAndGet(), forOwner);

		if (!contactStreams.containsKey(contactId))
			contactStreams.put(contactId, new LinkedList<>());

		contactStreams.get(contactId).add(stream);

		if (forOwner)
			this.notifyAll();

	}

	@Nullable
	public synchronized MailboxStorageStream getStream(ContactId contactId,
			boolean forOwner) {
		if (contactStreams.get(contactId) == null)
			return null;

		for (MailboxStorageStream s : contactStreams.get(contactId)) {
			if (!s.isPending() && (s.isForOwner() == forOwner)) {
				s.setPending(true);
				return s;
			}
		}
		return null;
	}

	public MailboxStorageStream getStreamForOwner() {
		for (LinkedList<MailboxStorageStream> streams : contactStreams
				.values()) {
			for (MailboxStorageStream s : streams)
				if (!s.isPending() && s.isForOwner()) {
					s.setPending(true);
					return s;
				}
		}
		return null;
	}

	public synchronized void deleteStream(MailboxStorageStream stream) {
		for (MailboxStorageStream s : contactStreams
				.get(stream.getContactId())) {
			if (s.getContactId() == stream.getContactId()) {
				contactStreams.get(stream.getContactId()).remove(s);
			}
		}
	}
}
