package org.briarproject.mailbox.android.overview;

import android.arch.lifecycle.ViewModelProviders;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.github.mikephil.charting.charts.PieChart;
import com.github.mikephil.charting.data.PieData;
import com.github.mikephil.charting.data.PieDataSet;
import com.github.mikephil.charting.data.PieEntry;
import com.github.mikephil.charting.formatter.PercentFormatter;
import com.github.mikephil.charting.utils.ColorTemplate;

import org.briarproject.bramble.api.event.Event;
import org.briarproject.bramble.api.nullsafety.MethodsNotNullByDefault;
import org.briarproject.mailbox.R;
import org.briarproject.mailbox.android.activity.ActivityComponent;
import org.briarproject.mailbox.android.fragment.BaseEventFragment;
import org.briarproject.mailbox.android.keyagreement.MailboxExchangeActivity;
import org.briarproject.mailbox.android.viewmodels.MailboxOwnerStatusViewModel;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import static android.view.View.GONE;
import static android.view.View.VISIBLE;
import static org.briarproject.mailbox.R.drawable.contact_offline;
import static org.briarproject.mailbox.R.drawable.contact_online;

@MethodsNotNullByDefault
@ParametersAreNonnullByDefault
public class OverviewFragment extends BaseEventFragment {

	public static final String TAG = OverviewFragment.class.getName();
	private static final Logger LOG = Logger.getLogger(TAG);

	private ImageView ownerStatus;
	private TextView ownerName, diskSpace;
	private PieChart pieChart;

	public static OverviewFragment newInstance() {
		OverviewFragment f = new OverviewFragment();
		Bundle bundle = new Bundle();
		f.setArguments(bundle);
		return f;
	}

	@Override
	public String getUniqueTag() {
		return TAG;
	}

	@Override
	public void injectFragment(ActivityComponent component) {
		component.inject(this);
	}

	@Nullable
	@Override
	public View onCreateView(LayoutInflater inflater,
			@Nullable ViewGroup container,
			@Nullable Bundle savedInstanceState) {
		getActivity().setTitle(R.string.overview);
		return inflater.inflate(R.layout.fragment_overview, container, false);
	}

	@Override
	public void onViewCreated(View view,
			@Nullable Bundle savedInstanceState) {
		super.onViewCreated(view, savedInstanceState);
		ownerStatus = view.findViewById(R.id.ownerStatus);
		ownerName = view.findViewById(R.id.ownerName);
		diskSpace = view.findViewById(R.id.diskSpace);
		pieChart = view.findViewById(R.id.piechart);
		pieChart.setUsePercentValues(true);
		List<PieEntry> yvalues = new ArrayList<>();


		yvalues.add(new PieEntry(50f, "Free"));
		yvalues.add(new PieEntry(25f, "Used by Mailbox"));
		yvalues.add(new PieEntry(25f, "Used by other apps"));
		PieDataSet dataSet = new PieDataSet(yvalues, "Disk Usage");
		dataSet.setColors(ColorTemplate.LIBERTY_COLORS);
		PieData data = new PieData(dataSet);
		data.setValueFormatter(new PercentFormatter());
		pieChart.setData(data);
		pieChart.invalidate();
		TextView ownerText = view.findViewById(R.id.ownerText);
		TextView emptyText = view.findViewById(R.id.emptyText);
		final MailboxOwnerStatusViewModel
				viewModel = ViewModelProviders.of(this)
				.get(MailboxOwnerStatusViewModel.class);
		viewModel.mailboxOwnerStatusLiveData.observe(this,
				mailboxOwnerStatus -> {
					if (mailboxOwnerStatus == null) {
						ownerName.setVisibility(GONE);
						ownerStatus.setVisibility(GONE);
						ownerText.setVisibility(GONE);
						diskSpace.setVisibility(GONE);
						pieChart.setVisibility(GONE);
						emptyText.setVisibility(VISIBLE);
					} else {
						emptyText.setVisibility(GONE);
						ownerText.setVisibility(VISIBLE);
						ownerName.setVisibility(VISIBLE);
						ownerName.setText(mailboxOwnerStatus.getName());
						diskSpace.setVisibility(VISIBLE);
						pieChart.setVisibility(VISIBLE);
						ownerStatus.setVisibility(VISIBLE);
						if (mailboxOwnerStatus.isOnline())
							ownerStatus.setImageResource(contact_online);
						else
							ownerStatus.setImageResource(contact_offline);
					}
				});
	}

	@Override
	public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
		inflater.inflate(R.menu.overview_actions, menu);
		super.onCreateOptionsMenu(menu, inflater);
	}

	@Override
	public void onStart() {
		super.onStart();
	}

	@Override
	public void eventOccurred(Event e) {

	}

	private void displayOwnerStatus() {
		runOnUiThreadUnlessDestroyed(() -> {

		});
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle presses on the action bar items
		switch (item.getItemId()) {
			case R.id.action_add_contact:
				Intent intent =
						new Intent(getContext(), MailboxExchangeActivity.class);
				startActivity(intent);
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}
	}
}
