package org.thoughtcrime.securesms.components.emoji;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.support.annotation.UiThread;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.AppCompatImageButton;
import android.util.AttributeSet;

import org.briarproject.mailbox.R;
import org.thoughtcrime.securesms.components.emoji.EmojiDrawer.EmojiDrawerListener;

import javax.annotation.Nullable;

@UiThread
public class EmojiToggle extends AppCompatImageButton
		implements EmojiDrawerListener {

	private final Drawable emojiToggle;
	private final Drawable imeToggle;

	public EmojiToggle(Context context) {
		this(context, null);
	}

	public EmojiToggle(Context context, @Nullable AttributeSet attrs) {
		this(context, attrs, 0);
	}

	public EmojiToggle(Context context, @Nullable AttributeSet attrs,
			int defStyle) {
		super(context, attrs, defStyle);

		emojiToggle = ContextCompat
				.getDrawable(getContext(), R.drawable.ic_emoji_toggle);
		imeToggle = ContextCompat
				.getDrawable(getContext(), R.drawable.ic_keyboard);
		setToEmoji();
	}

	public void setToEmoji() {
		setImageDrawable(emojiToggle);
	}

	public void setToIme() {
		setImageDrawable(imeToggle);
	}

	public void attach(EmojiDrawer drawer) {
		drawer.setDrawerListener(this);
	}

	@Override
	public void onShown() {
		setToIme();
	}

	@Override
	public void onHidden() {
		setToEmoji();
	}
}
